import { Controller, UsePipes } from '@nestjs/common';
import { QueryBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { RpcValidationPipe } from '@utils/pipes/rpc.validation-pipe';
import { FindAdByIdRequestDto } from './dtos/find-ad-by-id.request.dto';
import { FindAdByIdQuery } from '@modules/ad/core/queries/find-ad-by-id/find-ad-by-id.query';
import { AdResponseDto } from '../dtos/ad.response.dto';
import { AdEntity } from '@modules/ad/core/ad.entity';
import { AdMapper } from '@modules/ad/ad.mapper';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class FindAdByIdGrpcController {
  constructor(
    protected readonly mapper: AdMapper,
    private readonly queryBus: QueryBus,
  ) {}

  @GrpcMethod('AdsService', 'FindOneById')
  async findOnebyId(data: FindAdByIdRequestDto): Promise<AdResponseDto> {
    try {
      const ad: AdEntity = await this.queryBus.execute(
        new FindAdByIdQuery(data.id),
      );
      return this.mapper.toResponse(ad);
    } catch (e) {
      throw new RpcException({
        code: e.code,
        message: e.message,
      });
    }
  }
}
